# clustering.mod
# model for graph partitioning

param n >= 1, integer;  # number of vertices
param m >= 1, integer;  # number of edges
set V := 1..n;
set E within {V,V};

# edge weights
param c{E};  # edge weights
param I{E};  # edge inclusions

# vertex colours
param lambda{V};
param gamma{u in V, v in V : u != v} := 
  if (lambda[u] = lambda[v]) then 0 else 1;

# max number of clusters
param kmax default n;
set K := 1..kmax;

# original problem variables
var x{V,K} binary;
# linearization variables
var w{V,K,V,K} >= 0, <= 1;
# cluster existence variables
var z{K} binary;

# model
minimize intercluster :
  sum{k in K, l in K, (u,v) in E : k != l} I[u,v] * c[u,v] * w[u,k,v,l] +
  sum{k in K} z[k];

# constraints
subject to assignment {v in V} : sum{k in K} x[v,k] = 1;
subject to cardinality {k in K} : sum{v in V} x[v,k] <= ceil(card{V}/2);
subject to existence {k in K} : sum{v in V} x[v,k] >= z[k];
subject to diffcolours {u in V, v in V, k in K, l in K : u != v and k != l} : w[u,k,v,l] <= gamma[u,v];

# linearization constraints
subject to lin1 {u in V, v in V, h in K, k in K : (u,v) in E or (v,u) in E} : w[u,h,v,k] <= x[u,h];
subject to lin2 {u in V, v in V, h in K, k in K : (u,v) in E or (v,u) in E} : w[u,h,v,k] <= x[v,k];
subject to lin3 {u in V, v in V, h in K, k in K : (u,v) in E or (v,u) in E} : w[u,h,v,k] >= x[u,h] + x[v,k] - 1;

